#c7f0d8 and #43523d

require "app/nokia.rb"
require "app/ship.rb"
require "app/map.rb"
require "app/camera.rb"
require "app/player.rb"
require "app/title.rb"
require "app/menu.rb"
require "app/chat.rb"
require "app/combat.rb"
require "app/enemy.rb"
require "app/Vector2d.rb"

DEBUG = false
MUSIC_ENABLED = true
COLLISION_ENABLED = true
MAX_ENEMIES = 12

GAME_STATE = {
    title: 0,
    map: 1,
    town: 2,
    menu: 3,
    combat: 4,
    chat: 5,
    repair: 6,
    upgrade: 7,
    shop: 8,
    credits: 9,
    stats: 10,
    bombs: 11,
    ships: 12
}

def init args
    args.outputs.sounds << "sounds/port.ogg" if MUSIC_ENABLED
    args.state.gameState = GAME_STATE[:title]
    args.state.enemies = []

    $args = args
end

def tick args
    init(args) if (args.tick_count == 0)

    args.state.menu.title   ||= Title.new
    args.state.menu.pause   ||= PauseMenu.new
    args.state.menu.credits ||= Credits.new
    args.state.menu.town    ||= Town.new
    args.state.menu.repair  ||= Repair.new
    args.state.menu.upgrade ||= Upgrade.new
    args.state.menu.stats   ||= Stats.new
    args.state.menu.shop    ||= Shop.new
    args.state.menu.bombs   ||= BuyBombs.new
    args.state.menu.ships   ||= BuyShips.new

    args.state.player       ||= Player.new(args, 333, 356)
    args.state.camera       ||= Camera.new(args)
    args.state.map          ||= Map.new(args)
    args.state.combat       ||= Combat.new
    args.state.chat         ||= Chat.new

    args.state.enemies.push(Enemy.new(args)) if args.tick_count % 120 == 0 && args.state.enemies.length() < MAX_ENEMIES
    
    update(args)
    draw(args)
end

def update args
    case args.state.gameState
    when GAME_STATE[:title]
        args.state.menu.title.update(args)
    when GAME_STATE[:map]
        args.state.map.update(args)
        args.state.player.update(args)
        args.state.camera.update(args)
    when GAME_STATE[:town]
        args.state.menu.town.update(args)
    when GAME_STATE[:pause]
        args.state.menu.pause.update(args)
    when GAME_STATE[:credits]
        puts "Credits"
        args.state.menu.credits.update(args)
    when GAME_STATE[:repair]
        args.state.menu.repair.update(args)
    when GAME_STATE[:upgrade]
        args.state.menu.upgrade.update(args)
    when GAME_STATE[:shop]
        args.state.menu.shop.update(args)
    when GAME_STATE[:bombs]
        args.state.menu.bombs.update(args)
    when GAME_STATE[:ships]
        args.state.menu.ships.update(args)
    when GAME_STATE[:stats]
        args.state.menu.stats.update(args)
    when GAME_STATE[:combat]
        args.state.combat.update(args)
    when GAME_STATE[:chat]
        args.state.chat.update(args)
    else
        puts "Unhandled State: #{args.state.gameState}"
    end

    handleInput(args)
end

def draw args
    case args.state.gameState
    when GAME_STATE[:title]
        args.state.menu.title.draw(args)
    when GAME_STATE[:map]
        args.state.player.draw(args)
        args.state.map.draw(args)
    when GAME_STATE[:town]
        args.state.menu.town.draw(args)
    when GAME_STATE[:pause]
        args.state.menu.pause.draw(args)
    when GAME_STATE[:credits]
        args.state.menu.credits.draw(args)
    when GAME_STATE[:repair]
        args.state.menu.repair.draw(args)
    when GAME_STATE[:upgrade]
        args.state.menu.upgrade.draw(args)
    when GAME_STATE[:shop]
        args.state.menu.shop.draw(args)
    when GAME_STATE[:bombs]
        args.state.menu.bombs.draw(args)
    when GAME_STATE[:ships]
        args.state.menu.ships.draw(args)
    when GAME_STATE[:stats]
        args.state.menu.stats.draw(args)
    when GAME_STATE[:combat]
        args.state.combat.draw(args)
    when GAME_STATE[:chat]
        args.state.player.draw(args)
        args.state.map.draw(args)
        args.state.chat.draw(args)
    else
        puts "Unhandled State: #{args.state.gameState}"
    end
end

def handleInput args
    #DEBUG = !DEBUG if args.inputs.keyboard.key_down.d
    #COLLISION_ENABLED = !COLLISION_ENABLED if args.inputs.keyboard.key_down.zero
    
    args.state.map.minimap.toggle() if args.inputs.keyboard.key_down.m
    args.state.combat.start(args, Ship.new(args, 0, 0, "sm")) if args.inputs.keyboard.key_down.c

    if (args.inputs.keyboard.key_down.l)
        MUSIC_ENABLED = !MUSIC_ENABLED 

        if (MUSIC_ENABLED)
            args.outputs.sounds << "sounds/port.ogg"
        else
            args.gtk.stop_music()
        end
    end
end

def getDistance (p1, p2)
    x = p1[0] - p2[0]
    y = p1[1] - p2[1]
    return Math.sqrt((x * x) + (y * y))
end