require "app/mapdata.rb"

MAP_WIDTH = 420
MAP_HEIGHT = 384

TOWN = [280, 342, 40, 25]

class Map
    attr_accessor :x, :y, :minimap

    def initialize args
        @args = args
        @x ||= 0
        @y ||= 0
        @minimap = MiniMap.new(args, 62, 2)
    end

    def update args
        args.state.enemies.each do |e|
            e.update(args)
        end
    end

    def draw args
        args.nokia.sprites << {
            x: @x - args.state.camera.x,
            y: @y - args.state.camera.y,
            w: MAP_WIDTH,
            h: MAP_HEIGHT,
            path: "sprites/map.fw.png",
        }
        
        sprite_index = 0.frame_index(24, 6, true)

        args.nokia.sprites << {
            x: @x - args.state.camera.x,
            y: @y - args.state.camera.y,
            w: MAP_WIDTH,
            h: MAP_HEIGHT,
            path: "sprites/waves/waves_#{sprite_index}.png",
        }

        args.state.enemies.each do |e|
            e.draw(args)
        end

        @minimap.draw(args)

        if ([args.state.player.ship.x, args.state.player.ship.y].inside_rect? TOWN)
            drawOption(args, 1, "dock", true)
        end
    end
end

class MiniMap
    attr_accessor :visible, :x, :y, :start_tick

    def initialize (args, x, y)
        @args = args
        @x = x
        @y = y
        @visible = false
        @start_tick = 0
    end

    def draw args
        if (@visible)
            args.nokia.sprites << {
                x: @x,
                y: @y,
                w: 20,
                h: 20,
                path: "sprites/minimap.png",
            }

            sx = @x + (args.state.player.ship.x / 25).round + 1
            sy = @y + (args.state.player.ship.y / 22).round + 5
            frame = start_tick.frame_index(3, 24, true)
            
            args.nokia.labels << {
                x: sx,
                y: sy,
                text: [".", ".", ""][frame],
                font: NOKIA_FONT_8BIT_PATH,
                size_enum: NOKIA_FONT_8BIT_SM
            }
        end
    end

    def toggle 
        @start_tick = @args.tick_count
        @visible = !@visible
    end
end