def drawOption (args, y, text, selected)
    args.nokia.sprites << {
        x: 1,
        y: y,
        w: 82,
        h: 9,
        path: selected ? "sprites/menu-select2.png" : "sprites/menu-unselect.png"
    }

    if (selected)
        args.nokia.labels << {
            x: 4,
            y: y + 6,
            text: ">",
            size_enum: NOKIA_FONT_8BIT_SM,
            font: NOKIA_FONT_8BIT_PATH,
            r: NOKIA_LIGHT_R,
            g: NOKIA_LIGHT_G,
            b: NOKIA_LIGHT_B
        }
    end

    args.nokia.labels << {
        x: selected ? 9 : 4,
        y: y + 7,
        text: text,
        size_enum: NOKIA_FONT_ARK_SM,
        font: NOKIA_FONT_ARK_PATH,
        r: selected ? NOKIA_LIGHT_R : NOKIA_DARK_R,
        g: selected ? NOKIA_LIGHT_G : NOKIA_DARK_G,
        b: selected ? NOKIA_LIGHT_B : NOKIA_DARK_B
    }
end

BOMB_PRICE = 50
UPGRADE_PRICE = 50
UPGRADE_INCREMENT = 0.25
REPAIR_COST = 9

class PauseMenu
    attr_accessor :selected

    def open args
        args.state.gameState = GAME_STATE[:pause]
        @selected = 0
    end

    def update args
        @selected ||= 0

        @selected += 1 if args.inputs.keyboard.key_down.down && selected < 3
        @selected -= 1 if args.inputs.keyboard.key_down.up && selected > 0

        if (args.inputs.keyboard.key_down.enter || args.inputs.keyboard.key_down.x || args.inputs.keyboard.key_down.space)
            args.state.gameState = GAME_STATE[:map] if (@selected == 0)
            args.state.gameState = GAME_STATE[:stats] if (@selected == 1)
            args.state.gameState = GAME_STATE[:credits] if (@selected == 2)
            args.state.gameState = GAME_STATE[:title] if (@selected == 3)
        end

        args.state.gameState = GAME_STATE[:map] if args.inputs.keyboard.key_down.escape
    end

    def draw args
        args.nokia.labels << {
            x: 28,
            y: NOKIA_HEIGHT - 2,
            text: "Pause",
            size_enum: NOKIA_FONT_8BIT_SM,
            font: NOKIA_FONT_8BIT_PATH,
        }

        args.nokia.sprites << {
            x: 2,
            y: NOKIA_HEIGHT - 6,
            w: 80,
            h: 5,
            path: "sprites/pause.png"
        }

        drawOption(args, 31, "resume", @selected == 0)
        drawOption(args, 21, "ship", @selected == 1)
        drawOption(args, 11, "credits", @selected == 2)
        drawOption(args, 1, "quit to menu", @selected == 3)
    end
end

class Credits
    def update args
        args.state.gameState = GAME_STATE[:pause] if args.inputs.keyboard.key_down.escape || args.inputs.keyboard.key_down.enter || args.inputs.keyboard.key_down.x || args.inputs.keyboard.key_down.space
    end

    def draw args
        args.nokia.labels << {
            x: 22,
            y: NOKIA_HEIGHT - 2,
            text: "Credits",
            size_enum: NOKIA_FONT_8BIT_SM,
            font: NOKIA_FONT_8BIT_PATH,
        }

        args.nokia.sprites << {
            x: 2,
            y: NOKIA_HEIGHT - 6,
            w: 80,
            h: 5,
            path: "sprites/credits.png"
        }

        args.nokia.labels << {
            x: 19,
            y: 34,
            text: "@elCrabman",
            size_enum: NOKIA_FONT_ARK_SM,
            font: NOKIA_FONT_04B03_PATH,
        }

        args.nokia.labels << {
            x: 10,
            y: 24,
            text: "HypnoticOwl.com",
            size_enum: NOKIA_FONT_ARK_SM,
            font: NOKIA_FONT_04B03_PATH,
        }

        drawOption(args, 1, "back", true)
    end
end

class Town
    attr_accessor :selected

    def open args
        args.state.gameState = GAME_STATE[:town]
        @selected = 0
    end

    def update args
        @selected ||= 0

        @selected += 1 if args.inputs.keyboard.key_down.down && selected < 3
        @selected -= 1 if args.inputs.keyboard.key_down.up && selected > 0

        if (args.inputs.keyboard.key_down.enter || args.inputs.keyboard.key_down.x || args.inputs.keyboard.key_down.space)
            args.state.menu.repair.open(args) if (@selected == 0)
            args.state.menu.upgrade.open(args) if (@selected == 1)
            args.state.menu.shop.open(args) if (@selected == 2)
            args.state.gameState = GAME_STATE[:map] if (@selected == 3) 
        end

        args.state.gameState = GAME_STATE[:map] if args.inputs.keyboard.key_down.escape
    end

    def draw args
        args.nokia.labels << {
            x: 22,
            y: NOKIA_HEIGHT - 2,
            text: "Tortuga",
            size_enum: NOKIA_FONT_8BIT_SM,
            font: NOKIA_FONT_8BIT_PATH,
        }

        args.nokia.sprites << {
            x: 2,
            y: NOKIA_HEIGHT - 6,
            w: 80,
            h: 5,
            path: "sprites/credits.png"
        }

        drawOption(args, 31, "repair", @selected == 0)
        drawOption(args, 21, "upgrade", @selected == 1)
        drawOption(args, 11, "shop", @selected == 2)
        drawOption(args, 1, "back to ship", @selected == 3)
    end
end

class Repair
    attr_accessor :selected, :flashing, :timeout, :start_tick

    def open args
        args.state.gameState = GAME_STATE[:repair]
        @selected = 0
        @flashing = false
        @timeout = 0
    end

    def update args
        @selected ||= 0
        @flashing ||= false
        @timeout ||= 0

        @selected += 1 if args.inputs.keyboard.key_down.down && selected < 1
        @selected -= 1 if args.inputs.keyboard.key_down.up && selected > 0

        if @flashing
            if @timeout > 0
                @timeout -= 1
            else
                @flashing = false
            end
        end

        if (args.inputs.keyboard.key_down.enter || args.inputs.keyboard.key_down.x || args.inputs.keyboard.key_down.space)
            if @selected == 0
                cost = (args.state.player.ship.max_health - args.state.player.ship.health) * REPAIR_COST

                if (cost <= args.state.player.ship.gold)
                    args.state.player.ship.health = args.state.player.ship.max_health
                    args.state.player.ship.gold -= cost
                else
                    @start_tick = args.tick_count if !@flashing
                    @timeout = 100
                    @flashing = true
                end
            end

            args.state.gameState = GAME_STATE[:town] if @selected == 1
        end

        args.state.gameState = GAME_STATE[:town] if args.inputs.keyboard.key_down.escape
    end

    def draw args
        args.nokia.labels << {
            x: 25,
            y: NOKIA_HEIGHT - 2,
            text: "Repair",
            size_enum: NOKIA_FONT_8BIT_SM,
            font: NOKIA_FONT_8BIT_PATH,
        }

        args.nokia.sprites << {
            x: 2,
            y: NOKIA_HEIGHT - 6,
            w: 80,
            h: 5,
            path: "sprites/repair.png"
        }

        if (!@flashing || @start_tick.frame_index(2, 10, true) == 1)
            args.nokia.labels << {
                x: 4,
                y: 38,
                text: "Gold: #{args.state.player.ship.gold}",
                size_enum: NOKIA_FONT_ARK_SM,
                font: NOKIA_FONT_ARK_PATH,
            }
        end

        cost = (args.state.player.ship.max_health - args.state.player.ship.health) * REPAIR_COST

        args.nokia.labels << {
            x: 4,
            y: 30,
            text: "Cost: #{cost}",
            size_enum: NOKIA_FONT_ARK_SM,
            font: NOKIA_FONT_ARK_PATH,
        }

        drawOption(args, 11, "Repair", @selected == 0)
        drawOption(args, 1, "back", @selected == 1)
    end
end

class Upgrade
    attr_accessor :selected, :flashing, :timeout, :start_tick

    def open args
        args.state.gameState = GAME_STATE[:upgrade]
        @selected = 0
    end

    def update args
        @selected ||= 0
        @flashing ||= false
        @timeout ||= 0

        @selected += 1 if args.inputs.keyboard.key_down.down && selected < 2
        @selected -= 1 if args.inputs.keyboard.key_down.up && selected > 0

        if @flashing
            if @timeout > 0
                @timeout -= 1
            else
                @flashing = false
            end
        end

        if (args.inputs.keyboard.key_down.enter || args.inputs.keyboard.key_down.x || args.inputs.keyboard.key_down.space)
            if @selected == 0
                cost = (UPGRADE_PRICE * args.state.player.ship.rate_of_fire).round

                if (cost <= args.state.player.ship.gold)
                    args.state.player.ship.rate_of_fire += UPGRADE_INCREMENT
                    args.state.player.ship.gold -= cost
                else
                    @start_tick = args.tick_count if !@flashing
                    @timeout = 100
                    @flashing = true
                end
            end

            if @selected == 1
                cost = (args.state.player.ship.max_bombs * UPGRADE_PRICE).round

                if (cost <= args.state.player.ship.gold)
                    args.state.player.ship.bombs += 1
                    args.state.player.ship.max_bombs += 1
                    args.state.player.ship.gold -= cost
                else
                    @start_tick = args.tick_count if !@flashing
                    @timeout = 100
                    @flashing = true
                end
            end

            args.state.gameState = GAME_STATE[:town] if @selected == 2
        end

        args.state.gameState = GAME_STATE[:town] if args.inputs.keyboard.key_down.escape
    end

    def draw args
        <<-DOC
        args.nokia.labels << {
            x: 22,
            y: NOKIA_HEIGHT - 2,
            text: "Upgrade",
            size_enum: NOKIA_FONT_8BIT_SM,
            font: NOKIA_FONT_8BIT_PATH,
        }

        args.nokia.sprites << {
            x: 2,
            y: NOKIA_HEIGHT - 6,
            w: 80,
            h: 5,
            path: "sprites/credits.png"
        }
        DOC

        if (!@flashing || @start_tick.frame_index(2, 10, true) == 1)
            args.nokia.labels << {
                x: 4,
                y: 46,
                text: "Gold: #{args.state.player.ship.gold}",
                size_enum: NOKIA_FONT_ARK_SM,
                font: NOKIA_FONT_ARK_PATH,
            }
        end

        args.nokia.labels << {
            x: 4,
            y: 38,
            text: "B:#{args.state.player.ship.bombs}/#{args.state.player.ship.max_bombs}",
            size_enum: NOKIA_FONT_ARK_SM,
            font: NOKIA_FONT_ARK_PATH,
        }

        args.nokia.labels << {
            x: NOKIA_WIDTH / 2 - 2,
            y: 38,
            text: "S:#{args.state.player.ship.rate_of_fire.round(2)}",
            size_enum: NOKIA_FONT_ARK_SM,
            font: NOKIA_FONT_ARK_PATH,
        }

        drawOption(args, 21, "Speed (#{(args.state.player.ship.rate_of_fire * UPGRADE_PRICE).round})", @selected == 0)
        drawOption(args, 11, "Bombs (#{(args.state.player.ship.max_bombs * UPGRADE_PRICE).round})", @selected == 1)
        drawOption(args, 1, "back", @selected == 2)
    end
end

class Shop
    attr_accessor :selected

    def open args
        args.state.gameState = GAME_STATE[:shop]
        @selected = 0
    end

    def update args
        @selected ||= 0

        @selected += 1 if args.inputs.keyboard.key_down.down && selected < 2
        @selected -= 1 if args.inputs.keyboard.key_down.up && selected > 0

        if (args.inputs.keyboard.key_down.enter || args.inputs.keyboard.key_down.x || args.inputs.keyboard.key_down.space)
            args.state.menu.bombs.open(args) if (@selected == 0)
            args.state.menu.ships.open(args) if (@selected == 1)
            args.state.gameState = GAME_STATE[:town] if (@selected == 2)
        end

        args.state.gameState = GAME_STATE[:town] if args.inputs.keyboard.key_down.escape
    end

    def draw args
        args.nokia.labels << {
            x: 31,
            y: NOKIA_HEIGHT - 2,
            text: "Shop",
            size_enum: NOKIA_FONT_8BIT_SM,
            font: NOKIA_FONT_8BIT_PATH,
        }

        args.nokia.sprites << {
            x: 2,
            y: NOKIA_HEIGHT - 6,
            w: 80,
            h: 5,
            path: "sprites/stats.png"
        }

        drawOption(args, 21, "Bombs", @selected == 0)
        drawOption(args, 11, "Ships", @selected == 1)
        drawOption(args, 1, "back", @selected == 2)

    end
end

class BuyBombs
    attr_accessor :selected, :flashing, :timeout, :start_tick

    def open args
        args.state.gameState = GAME_STATE[:bombs]
        @selected = 0
        @flashing = false
        @timeout = 0
    end

    def update args
        @selected ||= 0
        @flashing ||= false
        @timeout ||= 0

        @selected += 1 if args.inputs.keyboard.key_down.down && selected < 2
        @selected -= 1 if args.inputs.keyboard.key_down.up && selected > 0

        if @flashing
            if @timeout > 0
                @timeout -= 1
            else
                @flashing = false
            end
        end

        if (args.inputs.keyboard.key_down.enter || args.inputs.keyboard.key_down.x || args.inputs.keyboard.key_down.space)
            if @selected == 0 && (args.state.player.ship.bombs < args.state.player.ship.max_bombs)
                cost = BOMB_PRICE

                if (cost <= args.state.player.ship.gold)
                    args.state.player.ship.bombs += 1
                    args.state.player.ship.gold -= cost
                else
                    @start_tick = args.tick_count if !@flashing
                    @timeout = 100
                    @flashing = true
                end
            end

            if @selected == 1 && (args.state.player.ship.bombs < args.state.player.ship.max_bombs)
                cost = (args.state.player.ship.max_bombs - args.state.player.ship.bombs) * BOMB_PRICE

                if (cost <= args.state.player.ship.gold)
                    args.state.player.ship.bombs = args.state.player.ship.max_bombs
                    args.state.player.ship.gold -= cost
                else
                    @start_tick = args.tick_count if !@flashing
                    @timeout = 100
                    @flashing = true
                end
            end

            args.state.gameState = GAME_STATE[:shop] if @selected == 2
        end

        args.state.gameState = GAME_STATE[:shop] if args.inputs.keyboard.key_down.escape
    end

    def draw args
        args.nokia.labels << {
            x: 28,
            y: NOKIA_HEIGHT - 2,
            text: "Bombs",
            size_enum: NOKIA_FONT_8BIT_SM,
            font: NOKIA_FONT_8BIT_PATH,
        }

        args.nokia.sprites << {
            x: 2,
            y: NOKIA_HEIGHT - 6,
            w: 80,
            h: 5,
            path: "sprites/pause.png"
        }

        if (!@flashing || @start_tick.frame_index(2, 10, true) == 1)
            args.nokia.labels << {
                x: NOKIA_WIDTH / 2,
                y: 38,
                text: "G:#{args.state.player.ship.gold}",
                size_enum: NOKIA_FONT_ARK_SM,
                font: NOKIA_FONT_ARK_PATH,
            }
        end

        args.nokia.labels << {
            x: 4,
            y: 38,
            text: "B:#{args.state.player.ship.bombs}/#{args.state.player.ship.max_bombs}",
            size_enum: NOKIA_FONT_ARK_SM,
            font: NOKIA_FONT_ARK_PATH,
        }

        fill = args.state.player.ship.max_bombs - args.state.player.ship.bombs

        drawOption(args, 21, "Buy (#{BOMB_PRICE})", @selected == 0)
        drawOption(args, 11, "Fill (#{fill*BOMB_PRICE})", @selected == 1)
        drawOption(args, 1, "back", @selected == 2)
    end
end

class BuyShips
    attr_accessor :selected, :flashing, :timeout, :start_tick

    def open args
        args.state.gameState = GAME_STATE[:ships]
        @selected = 0
        @flashing = false
        @timeout = 0
    end

    def update args
        @selected ||= 0
        @flashing ||= false
        @timeout ||= 0

        @selected += 1 if args.inputs.keyboard.key_down.down && selected < 1
        @selected -= 1 if args.inputs.keyboard.key_down.up && selected > 0

        if @flashing
            if @timeout > 0
                @timeout -= 1
            else
                @flashing = false
            end
        end

        if (args.inputs.keyboard.key_down.enter || args.inputs.keyboard.key_down.x || args.inputs.keyboard.key_down.space)
            if @selected == 0
                cost = 1000
                model = "md" if args.state.player.ship.model == "sm"
                model = "lg" if args.state.player.ship.model == "md"

                if (model)
                    if (cost <= args.state.player.ship.gold)
                        ship = Ship.new(args, args.state.player.ship.x, args.state.player.ship.y - 2, model, true)
                        ship.gold = args.state.player.ship.gold
                        args.state.player.ship = ship
                        args.state.player.ship.gold -= cost
                    else
                        @start_tick = args.tick_count if !@flashing
                        @timeout = 100
                        @flashing = true
                    end
                end
            end

            args.state.gameState = GAME_STATE[:shop] if @selected == 1
        end

        args.state.gameState = GAME_STATE[:shop] if args.inputs.keyboard.key_down.escape
    end

    def draw args
        args.nokia.labels << {
            x: 28,
            y: NOKIA_HEIGHT - 2,
            text: "Ships",
            size_enum: NOKIA_FONT_8BIT_SM,
            font: NOKIA_FONT_8BIT_PATH,
        }

        args.nokia.sprites << {
            x: 2,
            y: NOKIA_HEIGHT - 6,
            w: 80,
            h: 5,
            path: "sprites/pause.png"
        }

        if (!@flashing || @start_tick.frame_index(2, 10, true) == 1)
            args.nokia.labels << {
                x: 4,
                y: 27,
                text: "Gold: #{args.state.player.ship.gold}",
                size_enum: NOKIA_FONT_ARK_SM,
                font: NOKIA_FONT_ARK_PATH,
            }
        end

        # Ships
        model = "md" if args.state.player.ship.model == "sm"
        model = "lg" if args.state.player.ship.model == "md"

        if (model)
            args.nokia.sprites << {
                x: (NOKIA_WIDTH / 2) - ((model == "lg" ? 20 : 14) / 2) - 1,
                y: 30,
                w: model == "lg" ? 20 : 14,
                h: model == "lg" ? 10 :9,
                path: "sprites/ships/ship-#{model}-0.png",
                flip_horizontally: true
            }

            price = model == "lg" ? 10000 : 1000
        end

        drawOption(args, 11, model ? "Buy (#{price})" : "Sold out", @selected == 0)
        drawOption(args, 1, "back", @selected == 1)
    end
end

class Stats
    def open args
        args.state.gameState = GAME_STATE[:stats]
    end

    def update args
        args.state.gameState = GAME_STATE[:pause] if (args.inputs.keyboard.key_down.enter || args.inputs.keyboard.key_down.x || args.inputs.keyboard.key_down.space || args.inputs.keyboard.key_down.escape)
    end

    def draw args
        args.nokia.labels << {
            x: 31,
            y: NOKIA_HEIGHT - 2,
            text: "Ship",
            size_enum: NOKIA_FONT_8BIT_SM,
            font: NOKIA_FONT_8BIT_PATH,
        }

        args.nokia.sprites << {
            x: 2,
            y: NOKIA_HEIGHT - 6,
            w: 80,
            h: 5,
            path: "sprites/stats.png"
        }

        y = 39
        yd = 7

        args.nokia.labels << {
            x: 4,
            y: y,
            text: "Gold:  #{args.state.player.ship.gold}",
            size_enum: NOKIA_FONT_ARK_SM,
            font: NOKIA_FONT_ARK_PATH,
        }

        y -= yd

        args.nokia.labels << {
            x: 4,
            y: y,
            text: "Ship:  #{args.state.player.ship.health}/#{args.state.player.ship.max_health}",
            size_enum: NOKIA_FONT_ARK_SM,
            font: NOKIA_FONT_ARK_PATH,
        }

        y -= yd

        args.nokia.labels << {
            x: 4,
            y: y,
            text: "Bombs: #{args.state.player.ship.bombs}/#{args.state.player.ship.max_bombs}",
            size_enum: NOKIA_FONT_ARK_SM,
            font: NOKIA_FONT_ARK_PATH,
        }

        y -= yd

        args.nokia.labels << {
            x: 4,
            y: y,
            text: "Speed: #{args.state.player.ship.rate_of_fire.round(2)}",
            size_enum: NOKIA_FONT_ARK_SM,
            font: NOKIA_FONT_ARK_PATH,
        }

        drawOption(args, 1, "back", true)
    end
end